@@ -29,12 +29,14 @@ import fnmatch
 import hawkey
 from dnf.cli import commands
 from dnf.cli.option_parser import OptionParser
-from dnf.i18n import _
+from dnf.i18n import _, exact_width
 from dnf.pycomp import unicode
 
+
 def _maxlen(iterable):
     """Return maximum length of items in a non-empty iterable."""
-    return max(len(item) for item in iterable)
+    return max(exact_width(item) for item in iterable)
+
 
 class UpdateInfoCommand(commands.Command):
     """Implementation of the UpdateInfo command."""
@@ -94,6 +96,12 @@ class UpdateInfoCommand(commands.Command):
         output_format.add_argument("--info", dest='_spec_action', const='info',
                                    action='store_const',
                                    help=_('show info of advisories'))
+        parser.add_argument("--with-cve", dest='with_cve', default=False,
+                            action='store_true',
+                            help=_('show only advisories with CVE reference'))
+        parser.add_argument("--with-bz", dest='with_bz', default=False,
+                            action='store_true',
+                            help=_('show only advisories with bugzilla reference'))
         parser.add_argument('spec', nargs='*', metavar='SPEC',
                             choices=cmds, default=cmds[0],
                             action=OptionParser.PkgNarrowCallback,
@@ -114,14 +122,47 @@ class UpdateInfoCommand(commands.Command):
         if self.opts._availability:
             self.opts.availability = self.opts._availability
         else:
+            # yum compatibility - search for all|available|installed|updates in spec[0]
             if not self.opts.spec or self.opts.spec[0] not in self.availabilities:
                 self.opts.availability = self.availability_default
             else:
                 self.opts.availability = self.opts.spec.pop(0)
 
+        # filtering by advisory types (security/bugfix/enhancement/newpackage)
+        self.opts._advisory_types = set()
+        if self.opts.bugfix:
+            self.opts._advisory_types.add(hawkey.ADVISORY_BUGFIX)
+        if self.opts.enhancement:
+            self.opts._advisory_types.add(hawkey.ADVISORY_ENHANCEMENT)
+        if self.opts.newpackage:
+            self.opts._advisory_types.add(hawkey.ADVISORY_NEWPACKAGE)
+        if self.opts.security:
+            self.opts._advisory_types.add(hawkey.ADVISORY_SECURITY)
+
+        # yum compatibility - yum accepts types also as positional arguments
+        if self.opts.spec:
+            spec = self.opts.spec.pop(0)
+            if spec == 'bugfix':
+                self.opts._advisory_types.add(hawkey.ADVISORY_BUGFIX)
+            elif spec == 'enhancement':
+                self.opts._advisory_types.add(hawkey.ADVISORY_ENHANCEMENT)
+            elif spec in ('security', 'sec'):
+                self.opts._advisory_types.add(hawkey.ADVISORY_SECURITY)
+            elif spec == 'newpackage':
+                self.opts._advisory_types.add(hawkey.ADVISORY_NEWPACKAGE)
+            elif spec in ('bugzillas', 'bzs'):
+                self.opts.with_bz = True
+            elif spec == 'cves':
+                self.opts.with_cve = True
+            else:
+                self.opts.spec.insert(0, spec)
+
+        if self.opts.advisory:
+            self.opts.spec.extend(self.opts.advisory)
+
+
     def run(self):
         """Execute the command with arguments."""
-        mixed = False
         if self.opts.availability == 'installed':
             apkg_adv_insts = self.installed_apkg_adv_insts(self.opts.spec)
             description = _('installed')
@@ -129,7 +170,6 @@ class UpdateInfoCommand(commands.Command):
             apkg_adv_insts = self.updating_apkg_adv_insts(self.opts.spec)
             description = _('updates')
         elif self.opts.availability == 'all':
-            mixed = True
             apkg_adv_insts = self.all_apkg_adv_insts(self.opts.spec)
             description = _('all')
         else:
@@ -137,9 +177,9 @@ class UpdateInfoCommand(commands.Command):
             description = _('available')
 
         if self.opts.spec_action == 'list':
-            self.display_list(apkg_adv_insts, mixed)
+            self.display_list(apkg_adv_insts)
         elif self.opts.spec_action == 'info':
-            self.display_info(apkg_adv_insts, mixed)
+            self.display_info(apkg_adv_insts)
         else:
             self.display_summary(apkg_adv_insts, description)
 
@@ -150,75 +190,73 @@ class UpdateInfoCommand(commands.Command):
         return len(q) > 0
 
     def _advisory_matcher(self, advisory):
+        if not self.opts._advisory_types \
+                and not self.opts.spec \
+                and not self.opts.severity \
+                and not self.opts.bugzilla \
+                and not self.opts.cves \
+                and not self.opts.with_cve \
+                and not self.opts.with_bz:
+            return True
+        if advisory.type in self.opts._advisory_types:
+            return True
+        if any(fnmatch.fnmatchcase(advisory.id, pat) for pat in self.opts.spec):
+            return True
         if self.opts.severity and advisory.severity in self.opts.severity:
             return True
         if self.opts.bugzilla and any([advisory.match_bug(bug) for bug in self.opts.bugzilla]):
             return True
         if self.opts.cves and any([advisory.match_cve(cve) for cve in self.opts.cves]):
             return True
+        if self.opts.with_cve:
+            if any([ref.type == hawkey.REFERENCE_CVE for ref in advisory.references]):
+                return True
+        if self.opts.with_bz:
+            if any([ref.type == hawkey.REFERENCE_BUGZILLA for ref in advisory.references]):
+                return True
         return False
 
-    def _apackage_advisory_installeds(self, pkgs_query, cmptype, specs):
+    def _apackage_advisory_installed(self, pkgs_query, cmptype, specs):
         """Return (adv. package, advisory, installed) triplets."""
-        specs_types = set()
-        specs_patterns = set()
-        for spec in specs:
-            if spec == 'bugfix':
-                specs_types.add(hawkey.ADVISORY_BUGFIX)
-            elif spec == 'enhancement':
-                specs_types.add(hawkey.ADVISORY_ENHANCEMENT)
-            elif spec in ('security', 'sec'):
-                specs_types.add(hawkey.ADVISORY_SECURITY)
-            elif spec == 'newpackage':
-                specs_types.add(hawkey.ADVISORY_NEWPACKAGE)
-            else:
-                specs_patterns.add(spec)
-
-        if self.opts.bugfix:
-            specs_types.add(hawkey.ADVISORY_BUGFIX)
-        if self.opts.enhancement:
-            specs_types.add(hawkey.ADVISORY_ENHANCEMENT)
-        if self.opts.newpackage:
-            specs_types.add(hawkey.ADVISORY_NEWPACKAGE)
-        if self.opts.security:
-            specs_types.add(hawkey.ADVISORY_SECURITY)
-        if self.opts.advisory:
-            specs_patterns.update(self.opts.advisory)
-
         for apackage in pkgs_query.get_advisory_pkgs(cmptype):
             advisory = apackage.get_advisory(self.base.sack)
-            if not specs_types and not specs_patterns and not self.opts.severity and \
-                    not self.opts.bugzilla and not self.opts.cves:
-                advisory_match = True
-            else:
-                advisory_match = advisory.type in specs_types or \
-                    any(fnmatch.fnmatchcase(advisory.id, pat)
-                        for pat in specs_patterns) or \
-                    self._advisory_matcher(advisory)
+            advisory_match = self._advisory_matcher(advisory)
             apackage_match = any(fnmatch.fnmatchcase(apackage.name, pat)
-                                 for pat in specs_patterns)
+                                 for pat in self.opts.spec)
             if advisory_match or apackage_match:
                 installed = self._newer_equal_installed(apackage)
                 yield apackage, advisory, installed
 
+    def running_kernel_pkgs(self):
+        """Return query containing packages of currently running kernel"""
+        sack = self.base.sack
+        q = sack.query().filterm(empty=True)
+        kernel = sack.get_running_kernel()
+        if kernel:
+            q = q.union(sack.query().filterm(sourcerpm=kernel.sourcerpm))
+        return q
+
     def available_apkg_adv_insts(self, specs):
         """Return available (adv. package, adv., inst.) triplets"""
-        return self._apackage_advisory_installeds(
-            self.base.sack.query().installed(), hawkey.GT, specs)
+        # check advisories for the latest installed packages
+        q = self.base.sack.query().installed().latest(1)
+        # plus packages of the running kernel
+        q = q.union(self.running_kernel_pkgs().installed())
+        return self._apackage_advisory_installed(q, hawkey.GT, specs)
 
     def installed_apkg_adv_insts(self, specs):
         """Return installed (adv. package, adv., inst.) triplets"""
-        return self._apackage_advisory_installeds(
+        return self._apackage_advisory_installed(
             self.base.sack.query().installed(), hawkey.LT | hawkey.EQ, specs)
 
     def updating_apkg_adv_insts(self, specs):
         """Return updating (adv. package, adv., inst.) triplets"""
-        return self._apackage_advisory_installeds(
+        return self._apackage_advisory_installed(
             self.base.sack.query().filterm(upgradable=True), hawkey.GT, specs)
 
     def all_apkg_adv_insts(self, specs):
         """Return installed (adv. package, adv., inst.) triplets"""
-        return self._apackage_advisory_installeds(
+        return self._apackage_advisory_installed(
             self.base.sack.query().installed(), hawkey.LT | hawkey.EQ | hawkey.GT, specs)
 
     def _summary(self, apkg_adv_insts):
@@ -263,10 +301,10 @@ class UpdateInfoCommand(commands.Command):
         if self.base.conf.autocheck_running_kernel:
             self.cli._check_running_kernel()
 
-    def display_list(self, apkg_adv_insts, mixed):
+    def display_list(self, apkg_adv_insts):
         """Display the list of advisories."""
         def inst2mark(inst):
-            if not mixed:
+            if not self.opts.availability == 'all':
                 return ''
             elif inst:
                 return 'i '
@@ -282,8 +320,17 @@ class UpdateInfoCommand(commands.Command):
         nevra_inst_dict = dict()
         for apkg, advisory, installed in apkg_adv_insts:
             nevra = '%s-%s.%s' % (apkg.name, apkg.evr, apkg.arch)
-            nevra_inst_dict.setdefault((nevra, installed), dict())[advisory.id] = (
-                advisory.type, advisory.severity)
+            if self.opts.with_cve or self.opts.with_bz:
+                for ref in advisory.references:
+                    if ref.type == hawkey.REFERENCE_BUGZILLA and not self.opts.with_bz:
+                        continue
+                    elif ref.type == hawkey.REFERENCE_CVE and not self.opts.with_cve:
+                        continue
+                    nevra_inst_dict.setdefault((nevra, installed), dict())[ref.id] = (
+                        advisory.type, advisory.severity)
+            else:
+                nevra_inst_dict.setdefault((nevra, installed), dict())[advisory.id] = (
+                    advisory.type, advisory.severity)
 
         advlist = []
         # convert types to labels, find max len of advisory IDs and types
@@ -298,7 +345,7 @@ class UpdateInfoCommand(commands.Command):
         for (inst, aid, label, nevra) in advlist:
             print('%s%-*s %-*s %s' % (inst, idw, aid, tlw, label, nevra))
 
-    def display_info(self, apkg_adv_insts, mixed):
+    def display_info(self, apkg_adv_insts):
         """Display the details about available advisories."""
         arches = self.base.sack.list_arches()
         verbose = self.base.conf.verbose
@@ -329,7 +376,7 @@ class UpdateInfoCommand(commands.Command):
             if not verbose:
                 attributes[7] = None
                 attributes[8] = None
-            if mixed:
+            if self.opts.availability == 'all':
                 attributes[9] = [_('true') if installed else _('false')]
 
             width = _maxlen(labels)
@@ -341,7 +388,9 @@ class UpdateInfoCommand(commands.Command):
                 if atr_lines in (None, [None]):
                     continue
                 for i, line in enumerate(atr_lines):
-                    lines.append('%*s: %s' % (width, label if i == 0 else '', line))
+                    key = label if i == 0 else ''
+                    key_padding = width - exact_width(key)
+                    lines.append('%*s%s: %s' % (key_padding, "", key, line))
             return '\n'.join(lines)
 
         advisories = set()
